﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NUnit.Framework;
using UserManagement;

namespace UserManagementTests
{
    [TestFixture]
    public class UserControllerTests
    {
        private MockUserView mockView;
        private MockDataAccess mockDataAccess;
        private IUserView view;
        private UserController userController;

        [SetUp]
        public void Setup()
        {
            mockView = new MockUserView();
            mockDataAccess = new MockDataAccess();
            view = mockView;
            userController = new UserController(view, mockDataAccess);
        }

        [Test]
        public void UserControllerRegistersAsListenerForFieldUpdatedEvent()
        {
            MockUserView aMockView = new MockUserView();
            IUserView aView = aMockView;
            Assert.AreEqual(0, aMockView.NumberOfListenersForFieldUpdated);
            UserController controller = new UserController(aView, mockDataAccess);
            Assert.AreEqual(1, aMockView.NumberOfListenersForFieldUpdated);
        }

        [Test]
        public void UserControllerRegistersAsListenerForUserSubmittedEvent()
        {
            MockUserView aMockView = new MockUserView();
            IUserView aView = aMockView;
            Assert.AreEqual(0, aMockView.NumberOfListenersForUserSubmitted);
            UserController controller = new UserController(aView, mockDataAccess);
            Assert.AreEqual(1, aMockView.NumberOfListenersForUserSubmitted);
        }

        [Test]
        public void UserControllerCallsBackToIViewInvalidWhenFirstNameNull()
        {
            mockView.firstName = null;
            mockView.EnterField("FirstName");
            Assert.IsTrue(mockView.InvalidCallbackCalled);
        }

        [Test]
        public void UserControllerDoesNotCallBackWithValidFirstName()
        {
            mockView.firstName = "Cory";
            mockView.EnterField("FirstName");
            Assert.IsFalse(mockView.InvalidCallbackCalled);
        }

        [Test]
        public void UserControllerCallsBackToIViewInvalidWhenFirstNameNotNullAndLastNameNull()
        {
            mockView.firstName = "Cory";
            mockView.lastName = null;
            mockView.EnterField("LastName");
            Assert.IsTrue(mockView.InvalidCallbackCalled);
        }

        [Test]
        public void UserControllerCallsBackWhenDateOfBirthIsTomorrow()
        {
            mockView.dateOfBirth = DateTime.Now.AddDays(1);
            mockView.EnterField("DateOfBirth");
            Assert.IsTrue(mockView.InvalidCallbackCalled);
        }

        [Test]
        public void UserControllerSavesModelWhenValidAndUserSubmittedRaised()
        {
            mockView.firstName = "Justin";
            mockView.lastName = "Example";
            mockView.dateOfBirth = DateTime.Now.AddDays(-5);
            mockView.department = "QA";
            mockView.FireUserSubmittedEvent();
            Assert.IsTrue(mockDataAccess.SaveModelCalled);
        }

        [Test]
        public void PassingAnInvalidFieldToFieldEnteredTriggersInvalidCallback()
        {
            mockView.EnterField("SomeRandomString");
            Assert.IsTrue(mockView.InvalidCallbackCalled);
        }

        [Test]
        public void PassingAnInvalidDepartmentTriggersInvalidCallback()
        {
            mockView.department = "NotARealDepartment";
            mockView.EnterField("Department");
            Assert.IsTrue(mockView.InvalidCallbackCalled);
        }

        [Test]
        public void PassingAValidDepartmentDoesNotTriggerCallback()
        {
            mockView.department = "QA";
            mockView.EnterField("Department");
            Assert.IsFalse(mockView.InvalidCallbackCalled);
        }
    }

    public class MockDataAccess : IDataAccess
    {
        private bool saveModelCalled = false;

        public bool SaveModelCalled
        {
            get { return saveModelCalled; }
        }

        public void SaveModel(User user)
        {
            saveModelCalled = true;
        }
    }

    public class MockUserView : IUserView
    {
        bool invalidFieldCalled = false;
        public event FieldEnteredDelegate FieldEntered;
        public event UserSubmittedDelegate UserSubmitted;
        public void InvalidField(string fieldName, string message)
        {
            invalidFieldCalled = true;
        }

        public void EnterField(string fieldName)
        {
            FieldEntered(this, fieldName);
        }

        public string firstName;
        public string lastName;
        public DateTime dateOfBirth;
        public string department;

        public string FirstName { get { return firstName; } }
        public string LastName { get { return lastName; } }
        public DateTime DateOfBirth { get { return dateOfBirth; } }
        public string Department { get { return department; } }

        public void FireUserSubmittedEvent()
        {
            UserSubmitted(this);
        }
        public bool InvalidCallbackCalled
        {
            get { return invalidFieldCalled; }
        }

        public int NumberOfListenersForUserSubmitted
        {
            get
            {
                if (UserSubmitted == null)
                {
                    return 0;
                }
                else
                {
                    return UserSubmitted.GetInvocationList().Length;
                }
            }
        }

        public int NumberOfListenersForFieldUpdated
        {
            get 
            {
                if (FieldEntered == null)
                {
                    return 0;
                }
                else
                {
                    return FieldEntered.GetInvocationList().Length;
                }
            }
        }
    }
}
